package ssw.mj.test;

import junit.framework.TestCase;

import ssw.mj.symtab.Obj;
import ssw.mj.symtab.Struct;
import ssw.mj.symtab.Tab;

/**
 * Test cases for the symbol table.
 * 
 * @author AW
 */
public class SymTabTest extends TestCase {

	protected void setUp () {
		Tab.init();	
	}
	
	public void testUniverse () {
		Obj actual, expected, localVar;
		
		actual = Tab.find("int");
		expected = new Obj(Obj.Type, "int", Tab.intType, 0, 0, null);
		assertEquals("Universe 'int'", expected, actual);
		
		actual = Tab.find("char");
		expected = new Obj(Obj.Type, "char", Tab.charType, 0, 0, null);
		assertEquals("Universe 'char'", expected, actual);
		
		actual = Tab.find("null");
		expected = new Obj(Obj.Con, "null", Tab.nullType, 0, 0, null);
		assertEquals("Universe 'null'", expected, actual);
		
		actual = Tab.find("eol");
		expected = new Obj(Obj.Con, "eol", Tab.charType, 10, 0, null);
		assertEquals("Universe 'eol'", expected, actual);

		actual = Tab.find("chr");
		localVar = new Obj(Obj.Var, "i", Tab.intType, 0, 1, null);
		expected = new Obj(Obj.Meth, "chr", Tab.charType, 0, 1, localVar);
		assertEquals("Universe 'chr'", expected, actual);
		assertEquals("Universe 'chrObj'", expected, Tab.chrObj);
		assertTrue("Universe 'chrObj' found", actual == Tab.chrObj);

		actual = Tab.find("ord");
		localVar = new Obj(Obj.Var, "ch", Tab.charType, 0, 1, null);
		expected = new Obj(Obj.Meth, "ord", Tab.intType, 0, 1, localVar);
		assertEquals("Universe 'ord'", expected, actual);
		assertEquals("Universe 'ordObj'", expected, Tab.ordObj);
		assertTrue("Universe 'ordObj' found", actual == Tab.ordObj);
		
		actual = Tab.find("len");
		localVar = new Obj(Obj.Var, "arr", new Struct(Struct.Arr, Tab.noType), 0, 1, null);
		expected = new Obj(Obj.Meth, "len", Tab.intType, 0, 1, localVar);
		assertEquals("Universe 'ord'", expected, actual);		
		assertEquals("Universe 'lenObj'", expected, Tab.lenObj);
		assertTrue("Universe 'lenObj' found", actual == Tab.lenObj);
	}
	
	public void testInsertIntoUniverse () {
		Obj actual, expected;
		
		actual = Tab.insert(Obj.Prog, "MyClass", Tab.noType);
		expected = new Obj(Obj.Prog, "MyClass", Tab.noType, 0, 0, null);
		assertEquals("Insert 'MyClass'", expected, actual);

		actual = Tab.find("MyClass");
		assertEquals("Find 'MyClass'", expected, actual);
	}
	
	public void testInsertGlobal () {
		Tab.openScope();	// scope for global variables
		
		Obj actual = null, expected;

		actual = Tab.insert(Obj.Var, "x0", Tab.intType);
		expected = new Obj(Obj.Var, "x0", Tab.intType, 0, 0, null);
		assertEquals("First global", expected, actual);

		actual = Tab.find("x0");
		assertEquals("Find first global", expected, actual);
	}


		public void testInsertLocal () {
		Obj actual = null, expected;

		Tab.openScope();	// global scope
		Tab.openScope();	// local scope, level 1
		
		actual = Tab.insert(Obj.Var, "x", Tab.intType);
			
		expected = new Obj(Obj.Var, "x", Tab.intType, 0, 1, null);
		assertEquals("First valid local var on level 1", expected, actual);
		
		actual = Tab.find("x");
		assertEquals("Find first valid local var on level 1", expected, actual);
	}

	public static void main (String[] args) {
		junit.textui.TestRunner.run(SymTabTest.class);
	}
}
